//
//	gnuboxContainer.cpp - Gnubox main AppUi container cpp file.
//
//  Gnubox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Gnubox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Gnubox; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	Copyright (C) 2002 Mal Minhas.
//
//	Revision history:
//	=================
//	28/08/02:	First version of Gnubox started.

#include <eikenv.h>
#include "gnuboxContainer.h"
#include "gnuboxPhone.h" //for phone specific constants
#include "debug.h"
#include "cdb.h"

#ifndef BTREGISTERAO
//includes for UIQ BT REGISTER
#include <QBTselectdlg.h>
#endif 

#include <eiklabel.h>  // for example label control
#include <coemain.h>   // for iCoeEnv definition
#include <coecntrl.h>  // for ControlEnv() inline
#include <btextnotifiers.h> // for Bluetooth notifier
#include <btregistry.h>  // for accessing BT registry
#include <btmanclient.h>
#include <d32comm.h>

/**
   Information about CommDb
   ========================
   CommDb provides system-wide storage for communications-related
   settings. It holds information about Internet Access Providers
   (IAPs), Internet Service Providers (ISPs), GPRS, modems, locations,
   charge-cards, proxies and WAP settings

   CommDb stores the settings in a series of tables in a
   relational database, using the DBMS architecture.
   DBMS is a client/server architecture and provides for safe
   access by multiple clients.

   The CommDb API has two key concepts:
   <b>communications database</b>, and <b>table view</b>.

   The communications database interface allows clients to access
   the database, open its tables, and perform transactions. It is
   provided by CCommsDatabase and its base class CCommsDatabaseBase.

   Most database settings are read and written to by opening the
   appropriate table. Table access is provided by CCommsDbTableView.

   It is possible for several instances of most of the tables
   (e.g. where multiple IAPs have been set up) to exist in the database.
   The Connected preferences table sets a ranking by which such records
   are used. That table is accessed through its own view
   class CCommsDbConnectionPrefTableView.

   A view is simply a subset of the records (or rows) within a table,
   selected according to defined criteria. Tables, records, rows, columns,
   and so on, are DBMS concepts.

   Note that you can open two kinds of CCommsDatabase in 6.0 - EDatabaseTypeIAP
   and EDatabaseTypeISP.  The latter is deprecated in 6.1.

   For a database of type EDatabaseTypeIAP,
   tables are arranged as follows (from /commdb/scdb/cdbdata.cpp):

   LOCAL_D TTable const KTableList[]=
   {
   {IAP, KIAPTable},
   {DIAL_OUT_ISP, KDialOutISPTable},
   {DIAL_IN_ISP, KDialInISPTable},
   {WAP_ACCESS_POINT, KWAPAccessPointTable},
   {WAP_IP_BEARER, KWAPIPBearerTable},
   {WAP_SMS_BEARER, KWAPSMSBearerTable},
   {OUTGOING_GPRS, KGPRSTable},
   {INCOMING_GPRS, KGPRSTable},
   {MODEM, KModemTable},
   {LOCATION, KLocationTable},
   {CHARGECARD, KChargecardTable},
   {PROXIES, KProxiesTable},
   {CONNECTION_PREFERENCES, KConnectionPreferencesTable},
   {AGENT, KAgentTable},
   {GLOBAL_SETTINGS, KGlobalSettings},
   {BT_DEVICE_TABLE, KCBTDeviceTable},
   {BT_DEFAULT_TABLE, KCBTDefaultServiceTable},
   {DEFAULT_GPRS, KDefaultGPRSTable},
   {BT_SECURITY_TABLE, KCBTSecurityTable},
   {ACCESS_TYPE_TABLE, KAccessTypeTable},
   {0}
   };

   Note that the various defines here are in /commdb/inc/cdbcols.h:

   // tables...
   #define IAP												_S("IAP")
   #define DIAL_OUT_ISP									_S("DialOutISP")
   #define MODEM											_S("Modem")
   ...

   // DIAL_OUT_ISP table columns
   #define ISP_DESCRIPTION									_S("Description")					// text field for app to use to describe ISP
   #define ISP_TYPE										_S("Type")
   #define ISP_DEFAULT_TEL_NUM								_S("DefaultTelNum")
   ...

   // modem table columns
   #define MODEM_PORT_NAME									_S("PortName")
   #define MODEM_TSY_NAME									_S("TSYName")
   #define MODEM_CSY_NAME									_S("CSYName")
   #define MODEM_HANDSHAKING								_S("Handshaking")
   ...

   // Bluetooth table columns

   #define BT_DEV_ADD				_S("DeviceAddress")
   #define BT_DEV_DFLT_NAME		_S("DeviceDefaultName")
   #define BT_DEV_FRIENDLY_NAME	_S("DeviceFriendlyName")
   #define BT_DEVICE_CLASS			_S("DeviceClass")
   #define BT_LINKKEY_SIZE			_S("DeviceLinkKeySize")
   #define BT_LINKKEY				_S("DeviceLinkKey")
   #define BT_DEV_STATE			_S("DeviceStatus")
   #define BT_DEVICE_GLOBAL_SECURITY _S("DeviceGlobalSecurity")
   //
   #define BT_DFLT_SER_NAME		_S("DefaultServicename")
   #define BT_DFLT_SER_UUID		_S("DefaultServiceUID")
   #define BT_DFLT_DEV_ADD			_S("DefaultDeviceAddress")
   #define BT_SEC_UID				_S("BTSecurityUID")
   #define BT_SEC_DEVICE_SECURITY	_S("BTSecurityDevice")
   #define BT_DFLT_VIRTUAL_PORT    _S("DefaultVirtualPort")
   #define BT_DFLT_SECURITY_LEVEL  _S("DefaultSecurityLevel")

   Tables we are interested in are DIAL_OUT_ISP () and MODEM ().
   From (from /commdb/scdb/cdbdata.cpp):

   LOCAL_D TColumn const KDialOutISPTable[]=
   {
   {ISP_DESCRIPTION,EDbColText,0},
   ...
   {ISP_DEFAULT_TEL_NUM,EDbColText,0},
   ...
   {ISP_TYPE,EDbColUint32,0}
   ...
   {ISP_IF_NAME,EDbColText,TDbCol::ENotNull},
   ...
   };

   LOCAL_D TColumn const KModemTable[]=
   {
   ...
   {MODEM_PORT_NAME,EDbColText,0},
   {MODEM_TSY_NAME,EDbColText,0},
   {MODEM_CSY_NAME,EDbColText,0},
   ...
   {MODEM_HANDSHAKING,EDbColUint32,0},
   ...
   };

   LOCAL_D TColumn const KCBTDeviceTable[]=
   {
   ...
   {BT_DEV_ADD,EDbColText8,TDbCol::ENotNull},
   {BT_DEV_DFLT_NAME,EDbColLongText16,0},
   {BT_DEV_FRIENDLY_NAME,EDbColLongText16,0},
   {BT_DEVICE_CLASS,EDbColUint32,0},
   {BT_LINKKEY,EDbColText8,0},
   {BT_DEVICE_GLOBAL_SECURITY, EDbColUint32,0},
   {BT_DEV_STATE, EDbColUint32, TDbCol::ENotNull},
   ...
   };

   LOCAL_D TColumn const KCBTDefaultServiceTable[]=
   {
   ...	  	{BT_DFLT_SER_NAME,EDbColText16,0},
   {BT_DFLT_SER_UUID,EDbColText8,TDbCol::ENotNull},
   {BT_DFLT_VIRTUAL_PORT,EDbColUint32,TDbCol::ENotNull},
   {BT_DFLT_DEV_ADD,EDbColText8,TDbCol::ENotNull},
   {BT_DFLT_SECURITY_LEVEL,EDbColUint8,0},
   ...
   };

   LOCAL_D TColumn const KCBTSecurityTable[]=
   {
   ...
   {BT_DEV_ADD,EDbColText8,TDbCol::ENotNull},
   {BT_SEC_DEVICE_SECURITY,EDbColUint32,0},
   {BT_SEC_UID,EDbColUint32,0},
   ...
   };

   Note that you don't actually directly manipulate the BT fields in CommDB.
   The BTRegistry is your friend in this regard.  It acts as a facade over
   the CommDB.  Fine sentiment.  Now all we need is someone to do the same thing
   for the networking bits ;-)

   @TODO: Add more comments here on use cases for each option and IOP/testing.
*/

void CGnuboxContainer::ConstructL(const TRect& aRect, CGnuboxAppUi* aParent)
  /**
     CGnuboxContainer::ConstructL(const TRect& aRect)
     EPOC two phased constructor.
     Initialise to default settings.
  */
{
  FLOG(_L("CGnuboxContainer::ConstructL"));
  CreateWindowL();

  iParent = aParent;

  // This will add further text but it might take time so
  // we've hived it off to a separate active object...
  iLoader = CGnuboxAOLoader::NewL(this);
  //This will do the user interaction wait for bluetooth device selection
#ifdef BTREGISTERAO
  iBTRegister = CGnuboxAOBTRegister::NewL(this);
#else
  //no more need due to the UIQ BT device dialog is synchronous
  //maybe re-enable it for series 60 backport
#endif
  iLabel = new (ELeave) CEikLabel;
  iLabel->SetContainerWindowL(*this);

  // Set up the screen text with whatever the default bearer is...
  SetAndDrawScreenTextL(iParent->GetMode());  // MALM

  SetRect(aRect);

  ActivateL();
}

CGnuboxContainer::~CGnuboxContainer()
  /**
     Destructor
  */
{
  FLOG(_L("CGnuboxContainer::~CGnuboxContainer"));
  delete iLabel;
  iTextLines.Close();
  delete iLoader;
#ifdef BTREGISTERAO
  delete iBTRegister;
#endif
}

void CGnuboxContainer::SizeChanged()
  /**
     CGnuboxContainer::SizeChanged()
     Called by framework when the view size is changed.
     @TODO: Add control resize code etc here needed to resize label properly
  */
{
  FLOG(_L("CGnuboxContainer::SizeChanged"));
  iLabel->SetExtent(TPoint(10,5),iLabel->MinimumSize());
}

TInt CGnuboxContainer::CountComponentControls() const
  /**
     CGnuboxContainer::CountComponentControls() const
  */
{
  return 1; // return nbr of controls inside this container
}

CCoeControl* CGnuboxContainer::ComponentControl(TInt aIndex) const
  /**
     CGnuboxContainer::ComponentControl(TInt aIndex) const
  */
{
  switch ( aIndex )
    {
    case 0:
      return iLabel;
    default:
      return NULL;
    }
}

void CGnuboxContainer::Draw(const TRect& aRect) const
  /**
     CGnuboxContainer::Draw(const TRect& aRect) const
     Note that this redraws itself in different ways
     depending on what the current bearer is.
  */
{
  FLOG(_L("CGnuboxContainer::Draw"));
  DrawControls(aRect);
}

void CGnuboxContainer::HandleControlEventL(
                                           CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
  /**
     CGnuboxContainer::HandleControlEventL(
     CCoeControl* aControl,TCoeEvent aEventType)
     We are not interested in handling control events in
     this program (?).
  */
{
  // TODO: Add your control event handler code here
  FLOG(_L("CGnuboxContainer::HandleControlEventL"));
}


void CGnuboxContainer::SetModemL(TModemName m,TUint32 id) {
  CCommsDatabase * iCommDb=CCommsDatabase::NewL(EDatabaseTypeIAP);
  iCommDb->ShowHiddenRecords();
  
  CleanupStack::PushL(iCommDb);
  
  const TText* p=NULL;
  switch (m) {
  case EData:
    p=MODEM_DATA_FAX;
    break;
  case EPhone:
    p=MODEM_PHONE_SERVICES_SMS;
    break;
  }
  TInt err;
  TRAP(err,iCommDb->SetGlobalSettingL(TPtrC(p),id));
  if (err!=KErrNone) {
    iEikonEnv->InfoMsg(_L("Error setting global modem ID"));
    User::Leave(err);
  }

  CleanupStack::PopAndDestroy();//db
}


void CGnuboxContainer::SetDataModemL(TBool dir) {
  /*
    To be invoked once after installing GnuBox.  

    It sets the CommsDB global setting ModemForDataAndFax to the modem
    specified as KModemName (if that is found).

    This allows the splitting of SmsAndPhoneServices from the
    DataAndFax functionality, thus using gnubox to change the modem
    properties won't break SMS handling of the user.

    if dir is TRUE, the above apply. if dir is FALSE, it undoes the
    above. (i.e. sets the ModemForDataAndFax to whatever
    ModemForPhoneServicesAndSms is set). This allows to easily
    circumvent any changes gnubox may have done.

  */
  
  iEikonEnv->InfoMsg(_L("Switching DataAndFax modem"));
  FLOG(_L("**** CGnuboxContainer::SetDataModemL ****"));

  CCommsDatabase * iCommDb=CCommsDatabase::NewL(EDatabaseTypeIAP);
  FLOG(_L("SetDataModemL 1"));
  iCommDb->ShowHiddenRecords();
  FLOG(_L("SetDataModemL 2"));  
  CleanupStack::PushL(iCommDb);
  FLOG(_L("SetDataModemL 3"));
  TUint32 modemID;

  TInt err;
  if (dir) {
    FLOG(_L("SetDataModemL 4"));
    CCommsDbTableView* table=NULL;      
    //first find the id of the requested modem
    TRAP(err,table=iCommDb->OpenViewMatchingTextLC(TPtrC(MODEM_TABLE),TPtrC(COMMDB_NAME),KModemName); CleanupStack::Pop());
    FLOG(_L("SetDataModemL 5"));
    //  table=iCommDb->OpenTableLC(TPtrC(MODEM_TABLE),);
    if (err!=KErrNone) {
      TBuf<150> b;
      b.Format(_L("Modem not found. err=%d"),err);
      FLOG(b);
      iEikonEnv->InfoMsg(b);
      User::Leave(KErrNotFound);
    } else {
      CleanupStack::PushL(table);
    }
    FLOG(_L("Modem found"));
    User::LeaveIfError(table->GotoFirstRecord());
    FLOG(_L("SetDataModemL 6"));
    TRAP(err,table->ReadUintL(TPtrC(COMMDB_ID),modemID));
    FLOG(_L("Modem ID retrieved"));
    if (err!=KErrNone) {
      iEikonEnv->InfoMsg(_L("Error retrieving modem ID"));
      User::Leave(err);
    }
    FLOG(_L("SetDataModemL 7"));
    CleanupStack::PopAndDestroy();//table    
  } else {
    FLOG(_L("SetDataModemL 8"));
    iCommDb->GetGlobalSettingL(TPtrC(MODEM_PHONE_SERVICES_SMS),modemID);
  }
  FLOG(_L("SetDataModemL 9"));
  TRAP(err,iCommDb->SetGlobalSettingL(TPtrC(MODEM_DATA_FAX),modemID));
  FLOG(_L("SetDataModemL a"));
  if (err!=KErrNone) {
    iEikonEnv->InfoMsg(_L("Error setting global modem ID"));
    User::Leave(err);
  }
  FLOG(_L("Global modem ID set"));
  iCommDb->GetGlobalSettingL(TPtrC(MODEM_DATA_FAX),modemID);
  TBuf<30> iValue;
  FLOG(_L("Retr OK"));
  iValue.Format(_L("New modem ID %u"),modemID);
  iEikonEnv->InfoMsg(iValue);
  FLOG(iValue);
  FLOG(_L("Destroy start"));
  CleanupStack::PopAndDestroy();//db
}

void CGnuboxContainer::SwitchModeL(const TCfgEntry * aBearer)
  /**
     CGnuboxContainer::SwitchModeL(TConnectionMode aMode)
     Invoked whenever the user has decided to switch from
     one underlying connection mode to another.
  */
{
  iEikonEnv->InfoMsg(TPtrC(aBearer->iDispName));
  TBool okToUpdate=UpdateCommDbL(aBearer);
  if (okToUpdate)
    {// we are ok to switch mode
      iParent->SetMode(aBearer);
		
      ////Don't do the StoreRecord here - do it in Loader RunL

      // Then change the container control text strings to
      // ensure their correct updating on redraw
      SetAndDrawScreenTextL(aBearer);
    }
}

void CGnuboxContainer::DrawControls(const TRect& aRect) const
  /**
     CGnuboxContainer::DrawControls() const
     Main point at which AppUi controls are drawn
     Note that for 7650 width=176 and height=144
     @TODO: Need to scale the drawing of controls.
  */
{
  CWindowGc& gc = SystemGc();
  gc.SetPenStyle(CGraphicsContext::ENullPen);
  gc.SetBrushColor(iParent->GetMode()->iColor);
  gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
  gc.DrawRect(aRect);
  // Now we can draw screen elements
  //TPoint startpoint(0,iH*KHeightScalingFactor*3/5-40);
  TPoint startpoint(KXOffset,20);
  //TPoint endpoint(iW*KWidthScalingFactor,iH*KHeightScalingFactor*3/5-40);
  TPoint endpoint((KWidth-KXOffset),20);
  gc.SetPenStyle(CGraphicsContext::ESolidPen);
  TSize penSizeThicker(1,1);
  gc.SetPenSize(penSizeThicker);
  gc.SetPenColor(KRgbBlack);
  gc.DrawLine(startpoint,endpoint);

  //TFontSpec fs(_L("Arial"),20*7); // 9pt Arial
  TFontSpec fs(_L("Arial"),20*4); // 8pt Arial
  //TFontSpec fs(_L("Arial"),20*11); // 11pt Arial
  //TFontSpec fs(_L("Arial"),20*24); // 24pt Arial
  fs.iFontStyle=
    TFontStyle(EPostureUpright,EStrokeWeightNormal,EPrintPosNormal);
  CFont* fontUsed=ControlEnv()->CreateScreenFontL(fs);
  gc.UseFont(fontUsed);

  TRect textRect(KXOffset,23,(KWidth-KXOffset),KHeight);
  gc.DrawRect(textRect); // just to see our limits...

  // Now draw the lines of text - you must do this per redraw
  for(TInt count = 0; count < iTextLines.Count(); count++)
    gc.DrawText(iTextLines[count],TPoint(KTextOffset,35+(count*KLineGap)));

  gc.DiscardFont();
  ControlEnv()->ReleaseScreenFont(fontUsed);
  //
  gc.SetBrushStyle(CGraphicsContext::ENullBrush);
}

void CGnuboxContainer::SetAndDrawScreenTextL(const TCfgEntry * aBearer)
  /**
     CGnubox::SetAndDrawScreenTextL(TConnectionMode aMode)
     This function does two things:
     1) Changes the main label text
     2) Updates screen with new CommDb settings
     To do the latter, it invokes the iLoader active object.
  */
{
  FLOG(_L("CGnuboxContainer::SetScreenTextL"));
  TBuf<22> txtBuf;
  txtBuf.FillZ();
  txtBuf = TPtrC(aBearer->iDispName);
  txtBuf.SetMax();
  iLabel->SetTextL(txtBuf);
  iTextLines.Close();
  User::LeaveIfError(iTextLines.Append(_L("Loading CommDb")));
  User::LeaveIfError(iTextLines.Append(_L("settings...")));
  iLoader->Fire();
}

void CGnuboxContainer::ReadTextFieldL(CCommsDbTableView* aTable, const TDesC& aRecord)
  /**
     CGnuboxContainer::ReadTextFieldL(CCommsDbTableView* aTable, const TDesC& aRecord)
     Reads a particular field and appends it to iTextLines.
     @TODO: Need to add scrolling text window support - migrate to CTextView?
  */
{
  TBuf<KCommsDbSvrMaxFieldLength> printBuffer;
  TBuf<KCommsDbSvrMaxFieldLength-10> recBuffer;

  printBuffer=_L(" ");
  printBuffer.Append(aRecord);
  printBuffer.Append(_L(": "));
  aTable->ReadTextL(aRecord,recBuffer);

  // Need to store the value in dummyBuffer too...
  //If the array is already at the maximum size, delete the first line of text
  if (iTextLines.Count() >= KMaxTextLines)
    {
      iTextLines.Remove(0);
    }
  if (printBuffer.Length() + recBuffer.Length() > KMaxLineLength)
    {
      iTextLines.Append(printBuffer);
      TBuf<KCommsDbSvrMaxFieldLength> tempBuffer=_L("  ");
      tempBuffer.Append(recBuffer);
      iTextLines.Append(tempBuffer);
    }
  else
    {
      printBuffer.Append(recBuffer);
      iTextLines.Append(printBuffer);
    }

  if ((iParent->GetMode()->iID == CFGID_SAVE))
    {// Need to store these away because they are important
      // @TODO: Only do the following if it is different from current record...
      TBuf<KCommsDbSvrMaxFieldLength-10> currentBuffer;
      iParent->GetDocument()->RetrieveRecordL(aRecord,currentBuffer);
      if (currentBuffer != recBuffer)
        iParent->GetDocument()->StoreRecordL(aRecord,recBuffer);
    }
}

void CGnuboxContainer::LoadCommDbSettingsL()
  /**
     CGnuboxContainer::LoadCommDbSettingsL()
     This function loads the relevant CommDb settings

     Note that we store a couple of tables.
     Namely the MODEM table and the DIAL_OUT_ISP table.
     @TODO: Need to be pulling out the "Nokia CSD" IAP settings
     @TODO: There is a bug here relating to DIAL_OUT_ISP - what if we're GPRS??!!
  */
{
  FLOG(_L("**** CGnuboxContainer::LoadCommDbSettingsL ****"));

  // UPDATE THE CURRENT MODE IN - need to do this to pick up changes
  const TCfgEntry* mode=iParent->GetMode();
  iParent->GetDocument()->StoreRecordL(KModeTxt,TInt(mode->iID));

  // CREATE NEW COMMDB HANDLE
  if (iCommDb) 
    User::Leave(KErrInUse);

  iCommDb=CCommsDatabase::NewL(EDatabaseTypeIAP);
  iCommDb->ShowHiddenRecords();

  CleanupStack::PushL(iCommDb);

  // DIAL_OUT_ISP TABLE
  CCommsDbTableView* table=iCommDb->OpenTableLC(TPtrC(DIAL_OUT_ISP));
  TBuf<KCommsDbSvrMaxColumnNameLength> tableName;
  table->GetTableName(tableName);
  iTextLines.Append(tableName);
  TBool found=FindName(table, KISPName);
  if (!found) {
    TBuf<200> msg;
    msg.Format(_L("ISP '%S' not found"),&KISPName);
    iTextLines.Append(msg);
    User::Leave(1);
  }
  TInt ret;
  TRAP(ret, ReadTextFieldL(table,TPtrC(ISP_DESCRIPTION)));
  if (ret!=KErrNone) {
    TBuf<20> msg;
    msg.Format(_L("No desc %d"), ret);
    iTextLines.Append(msg);
    User::Leave(ret);
  }

  //ReadTextFieldL(table,TPtrC(ISP_TYPE));
  TRAP(ret, ReadTextFieldL(table,TPtrC(ISP_DEFAULT_TEL_NUM)));
  if (ret!=KErrNone) {
    TBuf<20> msg;
    msg.Format(_L("No telnum %d"), ret);
    iTextLines.Append(msg);
    User::Leave(ret);
  }
  CleanupStack::PopAndDestroy();  // table

  // MODEM
  table=iCommDb->OpenTableLC(TPtrC(MODEM_TABLE));
  table->GetTableName(tableName);
  iTextLines.Append(tableName);
  found=FindName(table, KModemName);
  if (!found) {
    TBuf<200> msg;
    msg.Format(_L("Modem '%S' not found"),&KModemName);
    iTextLines.Append(msg);
    User::Leave(1);
  }
  ReadTextFieldL(table,TPtrC(MODEM_PORT_NAME));
  ReadTextFieldL(table,TPtrC(MODEM_CSY_NAME));
  ReadTextFieldL(table,TPtrC(MODEM_TSY_NAME));
  CleanupStack::PopAndDestroy();  // table

  // GET RID OF COMMDB HANDLE
  CleanupStack::PopAndDestroy();  // dbs
  iCommDb = NULL;

  // **** The (re)drawing bit ****

  //TRect textRect(KXOffset,23,(KWidth-KXOffset),KHeight);
  //SetRect(textRect);
  DrawNow();
}

TBool CGnuboxContainer::FindName(CCommsDbTableView* table, const TDesC& name)
{
  TBool found=EFalse;
  while (!found && table->GotoNextRecord()==KErrNone) {
    TBuf<100> n;
    table->ReadTextL(TPtrC(COMMDB_NAME), n);
    if (!n.Compare(name) ) found=ETrue;
  }
  return found;
}


//! does the actual update in the commdb.
/**

   \param aTelNum is written to DIAL_OUT_ISP row KISPName column ISP_DEFAULT_TEL_NUM
   \param aDesc is written to DIAL_OUT_ISP row KISPName column ISP_DESCRIPTION
   \param aPort is written to MODEM_TABLE row KModemName column MODEM_PORT_NAME
   \param aCsy is written to MODEM_TABLE row KModemName column MODEM_CSY_NAME
   \param aTsy is written to MODEM_TABLE row KModemName column MODEM_TSY_NAME

   PITFALL: if you want to set something to empty string, you may have
   to rubout its default value. This is currently provided for aDescr
   and aTelNum.

*/
void CGnuboxContainer::WriteCommDbL(const TDesC & aTelNum, const TDesC& aDesc, const TDesC& aPort, const TDesC& aCsy, const TDesC& aTsy) {
  FLOG(_L("**** CGnuboxContainer::WriteCommDbL ****"));

  if(iCommDb)
    User::Leave(KErrInUse);
  iCommDb=CCommsDatabase::NewL(EDatabaseTypeIAP);
  iCommDb->ShowHiddenRecords();
  CleanupStack::PushL(iCommDb);
  // LOCK COMMDB - only needed for writes
  TInt ret = iCommDb->BeginTransaction();
  if (ret != 0) {
    iEikonEnv->InfoMsg(_L("Failed to lock CommDB"));
    User::Leave(KErrGeneral);
  }
  //rubout default values
  CCommsDbTableView* table=iCommDb->OpenTableLC(TPtrC(DIAL_OUT_ISP));
  User::LeaveIfError(table->GotoFirstRecord());
  User::LeaveIfError(table->UpdateRecord());
  table->SetNullL(TPtrC(ISP_DEFAULT_TEL_NUM));
  table->SetNullL(TPtrC(ISP_DESCRIPTION));
  User::LeaveIfError(table->PutRecordChanges());
  CleanupStack::PopAndDestroy();  // table

  //open ISP
  table=iCommDb->OpenViewMatchingTextLC(TPtrC(DIAL_OUT_ISP),TPtrC(COMMDB_NAME),KISPName);
  User::LeaveIfError(table->GotoFirstRecord());
  User::LeaveIfError(table->UpdateRecord());
  table->WriteTextL(TPtrC(ISP_DESCRIPTION),aDesc);
  table->WriteTextL(TPtrC(ISP_DEFAULT_TEL_NUM),aTelNum);
  User::LeaveIfError(table->PutRecordChanges());
  CleanupStack::PopAndDestroy();  // table

  //open modem
  table=iCommDb->OpenViewMatchingTextLC(TPtrC(MODEM_TABLE),TPtrC(COMMDB_NAME),KModemName);
  User::LeaveIfError(table->GotoFirstRecord());
  User::LeaveIfError(table->UpdateRecord());
  table->WriteTextL(TPtrC(MODEM_TSY_NAME),aTsy);
  table->WriteTextL(TPtrC(MODEM_CSY_NAME),aCsy);
  table->WriteTextL(TPtrC(MODEM_PORT_NAME),aPort);
  table->WriteUintL(TPtrC(MODEM_RATE),EBps460800);

  User::LeaveIfError(table->PutRecordChanges());
  CleanupStack::PopAndDestroy();  // table

  //commit changes
  ret=iCommDb->CommitTransaction();
  if(ret != KErrNone)
    {
      iEikonEnv->InfoMsg(_L("Failed to unlock CommDB"));
      FLOG(_L("Failed to update CommDB"));
      User::Leave(KErrGeneral);
    }
  // GET RID OF COMMDB HANDLE
  CleanupStack::PopAndDestroy();  // dbs
  FLOG(_L("Updated CommDB"));
  iCommDb = NULL;
} 





TBool CGnuboxContainer::UpdateCommDbL(const TCfgEntry * aBearer)
  /**
     TBool CGnubox::UpdateCommDbL(TConnectionMode aMode)
     Here we update the CommDb MODEM record to reflect the
     change in connection mode we have solicited.
     Currently there are five options:
     1) Direct PPP connection over Infrared
     2) Direct PPP connection over Bluetooth
     3) Dialup PPP connection over Infrared
     4) Dialup PPP connection over Bluetooth
     5) Default PPP over CSD/GPRS phone stack

     @TODO: Need to work out how to restore CSD/GPRS settings
  */
{
  FLOG(_L("**** CGnuboxContainer::UpdateCommDbL ****"));

  TDesC FTelN=KDirectTelNum;
  TBuf<20> telnum;
  if (!aBearer->iEmptyTelNo) {
    iParent->GetDocument()->RetrieveRecordL(KISPTelNumTxt,telnum);	
    FTelN=telnum;
  }
  if (aBearer->iSetBT) {
    TRAPD(err,SelectBluetoothDeviceL(aBearer->iBTProfile));
    if (err != 0) {
      FLOG(_L("Failed to select BT"));
      return FALSE;
    } else {
      FLOG(_L("Selected BT"));                  
    }
  }
  WriteCommDbL(FTelN,TPtrC(aBearer->iDescr),TPtrC(aBearer->iPortName),
               TPtrC(aBearer->iCSY),TPtrC(aBearer->iTSY));
  return TRUE;
}

void CGnuboxContainer::SelectBluetoothDeviceL(TUint32 aProfile)
  /**
     CGnuboxContainer::SelectBluetoothDeviceL()
     Deals with Bluetooth device selection.
  */
{
  FLOG(_L("CGnuboxContainer::SelectBluetoothDevice"));
  
#ifdef BTGREGISTERAO
  iBTRegister->StartDialog(aProfile);
  //All the job was moved to the Active Object gnuboxAOBTRegister
#else
  //UIQ BT select dialog
  TBTDevAddr remoteDevAddr(0);
  TBTDeviceName remoteDevName;
  TBTDeviceClass remoteDevClass;
  TInt serviceFilter = -1;
  CQBTUISelectDialog* btSelectDialog = new(ELeave) CQBTUISelectDialog
    (remoteDevAddr, remoteDevName, remoteDevClass,
     CQBTUISelectDialog::EQBTDeviceFilterAll, serviceFilter);
  if(btSelectDialog->LaunchSingleSelectDialogLD()) {
    TBTDeviceResponseParams p;
    p.SetDeviceAddress(remoteDevAddr);
    p.SetDeviceClass(remoteDevClass);
    p.SetDeviceName(remoteDevName);
    SetupCSYL(p,aProfile);
  } else {
    //we leave if user presses cancel
    User::Leave(0);
  }
#endif
}

void CGnuboxContainer::SetupCSYL(const TBTDeviceResponseParams &resp, TUint32 aProfile)
{
  iDevAddr = resp.BDAddr();
  iDevName = resp.DeviceName();
  FLOG(_L("1"));
  TBTCommPortSettings	settings;

  RNotifier n;
  User::LeaveIfError(n.Connect());
  FLOG(_L("2"));
  TRequestStatus s;
  TBuf<150> b;
  FLOG(_L("3"));
  TBuf<12> addrAsText;
  addrAsText.Zero();
  addrAsText.Format(_L("%02x%02x%02x%02x%02x%02x"),
                    iDevAddr[0], iDevAddr[1], iDevAddr[2], iDevAddr[3], iDevAddr[4], iDevAddr[5]);
  FLOG(_L("3b"));
  b.Zero();
  b.Format(_L("Connecting to %S"),&iDevName);
  FLOG(_L("4"));
  TInt butret;
  FLOG(_L("X"));
  n.Notify(b,_L("Do you want to require encryption?"),_L("Yes"),_L("No"),butret,s);
  FLOG(_L("Y"));
  User::WaitForRequest(s);
  FLOG(_L("Z"));
  n.Close();
  if (butret==0) {
    settings.SetEncryption();
  } else {
    settings.SetAuthentication();
  }

  settings.SetUUID(TUint(aProfile));  // The service - e.g. DUNP=0x1103
  settings.SetPort(5);				// n where BTCOMM::n
  settings.SetBTAddr(iDevAddr);


  FLOG(addrAsText);
  FLOG(_L("BTCOMM CSY Configured\n"));

  CBTRegistry* registry=CBTRegistry::NewLC();
  //  CleanupStack::PushL(registry);
  TInt err;

  TRAP(err,registry->SetDefaultCommPort(settings));
  if (err!=KErrNone) {
    TBuf<100> b;
    b.Format(_L("Setting BT registry: error %d "),err);
    FLOG(b);
    iEikonEnv->InfoMsg(b);
    User::Leave(err);
  }
  iEikonEnv->InfoMsg(_L("Set registry here"));
	
  FLOG(_L("BTCOMM CSY Configured\n"));
  CleanupStack::PopAndDestroy();
}




