/*

  gnuboxPhone.cpp

  this file holds the static settings database.

*/

#include "gnuboxPhone.h"
#include "gnubox.hrh"
#include "debug.h"
#define TEXTCAST(a) _S(a)

#define PHONETSY  TEXTCAST("ERIGSM")
#define BTCSY  TEXTCAST("BTCOMM")

#define COLOR_REDDISH  0xc8c8ff
#define COLOR_BLUISH   0xffc8c8
#define COLOR_GREENISH 0xc8ffc8
#define COLOR_WHITE    0xffffff

// enums
enum TBTProfile
  {
    ESerialPortProfile=0x1101,
    ELANAccess=0x1102,
    EDialupNetworking=0x1103,
  };

const PhoneCfg::TCfgEntry PhoneCfg::PHONE_SETTINGS[] = {
  { EGnuboxCmdAppCSD_GPRS,    //the gnubox command id to define.
    TEXTCAST("PPP over CSD/GPRS     "), //this will be shown in the first row of the display
    TEXTCAST("1box phone"),             //DIAL_OUT_ISP -> Description
    FALSE,                    //true if empty tel.no is required
    TEXTCAST("MUXCOMM::0"),             //MODEM -> PortName
    TEXTCAST("ECMUX"),                  //MODEM -> CsyName
    PHONETSY,                 //MODEM -> TsyName
    FALSE,                    //true if BT discovery is required
    0,                        //UUID of BT profile to use
    COLOR_WHITE
  },

  { EGnuboxCmdAppInfraredDirect,    //the gnubox command id to define.
    TEXTCAST("Direct over IR        "), //this will be shown in the first row of the display
    TEXTCAST("Direct 2box"),             //DIAL_OUT_ISP -> Description
    TRUE,                    //true if empty tel.no is required
    TEXTCAST("IRCOMM::0"),             //MODEM -> PortName
    TEXTCAST("IRCOMM"),                  //MODEM -> CsyName
    PHONETSY,                 //MODEM -> TsyName
    FALSE,                    //true if BT discovery is required
    0,                        //UUID of BT profile to use
    COLOR_REDDISH
  },

  { EGnuboxCmdAppBluetoothDirectMR,    //the gnubox command id to define.
    TEXTCAST("Direct over BT/Auto   "), //this will be shown in the first row of the display
    TEXTCAST("Direct 2box"),             //DIAL_OUT_ISP -> Description
    TRUE,                    //true if empty tel.no is required
    TEXTCAST("BTCOMM::0"),             //MODEM -> PortName
    BTCSY,                  //MODEM -> CsyName
    PHONETSY,                 //MODEM -> TsyName
    FALSE,                    //true if BT discovery is required
    0,                        //UUID of BT profile to use
    COLOR_BLUISH
  },

  { EGnuboxCmdAppBluetoothDirectSer,//the gnubox command id to define.
    TEXTCAST("Direct over BT/Serial "),  //this will be shown in the first row of the display
    TEXTCAST("Direct 2box"),             //DIAL_OUT_ISP -> Description
    TRUE,                      //true if empty tel.no is required
    TEXTCAST("BTCOMM::5"),               //MODEM -> PortName
    BTCSY,                   //MODEM -> CsyName
    PHONETSY,                  //MODEM -> TsyName
    TRUE,                      //true if BT discovery is required
    ESerialPortProfile,        //UUID of BT profile to use
    COLOR_BLUISH
  },

  { EGnuboxCmdAppBluetoothDirectLan,//the gnubox command id to define.
    TEXTCAST("Direct over BT/LANAcc "),  //this will be shown in the first row of the display
    TEXTCAST("Direct 2box"),             //DIAL_OUT_ISP -> Description
    TRUE,                      //true if empty tel.no is required
    TEXTCAST("BTCOMM::5"),               //MODEM -> PortName
    BTCSY,                   //MODEM -> CsyName
    PHONETSY,                  //MODEM -> TsyName
    TRUE,                      //true if BT discovery is required
    ELANAccess,                //UUID of BT profile to use
    COLOR_BLUISH
  },
  { EGnuboxCmdAppBluetoothDirectDun,//the gnubox command id to define.
    TEXTCAST("Direct over BT/Dialup "),  //this will be shown in the first row of the display
    TEXTCAST("Direct 2box"),             //DIAL_OUT_ISP -> Description
    TRUE,                      //true if empty tel.no is required
    TEXTCAST("BTCOMM::5"),               //MODEM -> PortName
    BTCSY,                   //MODEM -> CsyName
    PHONETSY,                  //MODEM -> TsyName
    TRUE,                      //true if BT discovery is required
    EDialupNetworking,         //UUID of BT profile to use
    COLOR_BLUISH
  },

  { EGnuboxCmdAppCableDirect, //the gnubox command id to define.
    TEXTCAST("Direct over Cable     "), //this will be shown in the first row of the display
    TEXTCAST("Direct 2box"),            //DIAL_OUT_ISP -> Description
    TRUE,                     //true if empty tel.no is required
    TEXTCAST("COMM::4"),                //MODEM -> PortName
    TEXTCAST("ECUART"),                 //MODEM -> CsyName
    PHONETSY,                 //MODEM -> TsyName
    FALSE,                    //true if BT discovery is required
    0,                        //UUID of BT profile to use
    COLOR_GREENISH
  },

  { 0
  }
};




const PhoneCfg::TCfgEntry * PhoneCfg::GetSettings(TInt aID) {
  const TCfgEntry * ce = PHONE_SETTINGS;
  while ((ce->iID) && (ce->iID!=aID)) {
    ce++;
  }
  if (ce->iID) {
    return ce;
  } else {
    return NULL;
  }

}

const PhoneCfg::TCfgEntry * PhoneCfg::GetDefaultSettings() {
  return PHONE_SETTINGS;
}

const PhoneCfg::TCfgEntry * PhoneCfg::GetSettingsWFallback(TInt aID) {
  const TCfgEntry * ce = PHONE_SETTINGS;
  while ((ce->iID) && (ce->iID!=aID)) {
    ce++;
  }
  if (ce->iID) {
    return ce;
  } else {
    TBuf<80> b;
    b.Format(_L("Unknown phone settings id %d requested."),aID);
    FLOG(b);
    return PHONE_SETTINGS;
  }
}
